/*
 * CalcParser.l
 *
 * Created on 30 de octubre de 2006, 22:29
 *
 * This program is the legal property of its developers, whose names are
 * listed with this program.  Please refer to the COPYRIGHT file distributed
 * with this source distribution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package icalc;
import icalc.calculation.*;

import java.util.*;
import java.io.*;

import icalc.calculation.value.*;

%%

%class CalcParserYylex
%byaccj

%{
    final static char END_CHAR = 0; 

    private File          file;
    private CalcParser    yyparser;
    private Vector<Value> memory;

    private int nline = 1;
    private int ncol  = 0;

    public CalcParserYylex(String expression, CalcParser yyparser, Vector<Value> memory) throws IOException, ParseException
    {
        this(new CalcParserReader(expression, END_CHAR));

        this.file     = null;
        this.yyparser = yyparser;
        this.memory   = memory;
    }

    public CalcParserYylex(File file, CalcParser yyparser) throws IOException, ParseException
    {
        this(new CalcParserReader(file, END_CHAR));

        this.file     = file;
        this.yyparser = yyparser;
        this.memory   = null;
    }

    public void moveCursor()
    {
        for (char c : yytext().toCharArray())
        {
            if (c == '\n')
            {
                nline ++;
                ncol = 0;
            }
            else
            {
                ncol ++;
            }
        }
    }

    public FilePosition getCurrentPosition()
    {
        return new FilePosition(file, nline, ncol);
    }

%}

SPC = [ \t\n\r]+

LINECOMMENT = \/\/ [^\n]* \n

INLINECOMMENT = "/*" .* "*/"

COMMENT = {LINECOMMENT} | {INLINECOMMENT}

TAILCOMMENT  = \/\/ [^\n]*

INT = [0-9]+

FLT = {INT}? "." {INT}?

STR1 = \' [^\0\']* ( \\ \' [^\0\']* )* \'

STR2 = \" [^\0\"]* ( \\ \" [^\0\"]* )* \"

STR = {STR1} | {STR2}

MEM = [\$]{INT}

NAME = [a-zA-Z_] [a-zA-Z0-9_]*

PARAM = {SPC} [^\0]*

HELP = ("help"  | "?")   {PARAM}? 

EXIT = "bye"             {PARAM}? 

CLS  = "cls"             {PARAM}? 

LCLA = "loadjar"         {PARAM}? 

LJAR = "loadclass"       {PARAM}? 

CLN  = ";" ( {SPC}? ";" )*

%%

{COMMENT}             { moveCursor(); }

{TAILCOMMENT}? \0     { moveCursor(); return CalcParser.ENDOFINPUT; }

"("    { moveCursor(); return CalcParser.PARENT; }

")"    { moveCursor(); return CalcParser.PARENT_END; }

"{"    { moveCursor(); return CalcParser.BRACE; }

"}"    { moveCursor(); return CalcParser.BRACE_END; }

"["    { moveCursor(); return CalcParser.BRACKET; }

"]"    { moveCursor(); return CalcParser.BRACKET_END; }

"="    { moveCursor(); return CalcParser.ASSIGN; }

"=="   { moveCursor(); return CalcParser.EQ; }

"!="   { moveCursor(); return CalcParser.NE; }

"<"    { moveCursor(); return CalcParser.LT; }

">"    { moveCursor(); return CalcParser.GT; }

"<="   { moveCursor(); return CalcParser.LE; }

">="   { moveCursor(); return CalcParser.GE; }

"||"   { moveCursor(); return CalcParser.OR; }

"&&"   { moveCursor(); return CalcParser.AND; }

"|"    { moveCursor(); return CalcParser.BOR; }

"^"    { moveCursor(); return CalcParser.BXOR; }

"&"    { moveCursor(); return CalcParser.BAND; }

"<<"   { moveCursor(); return CalcParser.LSHIFT; }

">>"   { moveCursor(); return CalcParser.RSHIFT; }

"~"    { moveCursor(); return CalcParser.BNEG; }

"!"    { moveCursor(); return CalcParser.NOT; }

"+"    { moveCursor(); return CalcParser.ADD; }

"-"    { moveCursor(); return CalcParser.SUB; }

"*"    { moveCursor(); return CalcParser.MULT; }

"/"    { moveCursor(); return CalcParser.DIV; }

"%"    { moveCursor(); return CalcParser.MOD; }
        
"**"   { moveCursor(); return CalcParser.POW; }

"++"   { moveCursor(); return CalcParser.INC; }

"--"   { moveCursor(); return CalcParser.DEC; }

"||="  { moveCursor(); return CalcParser.ORASSIGN; }

"&&="  { moveCursor(); return CalcParser.ANDASSIGN; }

"|="   { moveCursor(); return CalcParser.BORASSIGN; }

"^="   { moveCursor(); return CalcParser.BXORASSIGN; }

"&="   { moveCursor(); return CalcParser.BANDASSIGN; }

"<<="  { moveCursor(); return CalcParser.LSHIFTASSIGN; }

">>="  { moveCursor(); return CalcParser.RSHIFTASSIGN; }

"+="   { moveCursor(); return CalcParser.ADDASSIGN; }

"-="   { moveCursor(); return CalcParser.SUBASSIGN; }

"*="   { moveCursor(); return CalcParser.MULTASSIGN; }

"/="   { moveCursor(); return CalcParser.DIVASSIGN; }

"%="   { moveCursor(); return CalcParser.MODASSIGN; }
        
"**="  { moveCursor(); return CalcParser.POWASSIGN; }

","    { moveCursor(); return CalcParser.COMMA; }

"." | "->" { moveCursor(); return CalcParser.DOT; }

{CLN}  { moveCursor(); return CalcParser.COLON; }

"if"   { moveCursor(); return CalcParser.IF; }

"else" { moveCursor(); return CalcParser.ELSE; }

"do"   { moveCursor(); return CalcParser.DO; }

"while" { moveCursor(); return CalcParser.WHILE; }

"for"  { moveCursor(); return CalcParser.FOR; }

"function"  { moveCursor(); return CalcParser.FUNCTION; }

"return"  { moveCursor(); return CalcParser.RETURN; }

"break"  { moveCursor(); return CalcParser.BREAK; }

"def"  { moveCursor(); return CalcParser.DEF; }

"include" { moveCursor(); return CalcParser.INCLUDE; }

{HELP} { moveCursor(); yyparser.yylval = new CalcParserVal(yytext()); return CalcParser.HELPCMD;      }
{EXIT} { moveCursor(); yyparser.yylval = new CalcParserVal(yytext()); return CalcParser.EXITCMD;      }
{CLS}  { moveCursor(); yyparser.yylval = new CalcParserVal(yytext()); return CalcParser.CLEARCMD;     }
{LCLA} { moveCursor(); yyparser.yylval = new CalcParserVal(yytext()); return CalcParser.LOADJARCMD;   }
{LJAR} { moveCursor(); yyparser.yylval = new CalcParserVal(yytext()); return CalcParser.LOADCLASSCMD; }

{INT}  { moveCursor();
         yyparser.yylval = new CalcParserVal(yyparser.pushValue(Long.parseLong(yytext())));
	 return CalcParser.VALUE; }

{FLT}  { moveCursor();
         yyparser.yylval = new CalcParserVal(yyparser.pushValue(Double.parseDouble(yytext())));
	 return CalcParser.VALUE; }

{STR}  { moveCursor();
         String yytext = yytext();
         String s = "";

         boolean scaped = false;

         for (char c : yytext.substring(1, yytext.length()-1).toCharArray())
         {
           if (scaped)
           {
             s += c;
             scaped = false;
           }
           else
           {
             if (c == '\\')
               scaped = true;
             else
               s += c;
           }
         }
         
         yyparser.yylval = new CalcParserVal(yyparser.pushValue(s));
	 return CalcParser.VALUE; }

{MEM}  { moveCursor();

         if (memory == null)
         {
             yyparser.yyerror("Bad operation: Files can not access memory positions");
             return -1;
         }
         else
         {
             int index = Integer.parseInt(yytext().substring(1));

             if (index >= memory.size())
             {
               yyparser.yyerror("bad memory access '" + yytext() + "'");
               return -1;
             }
             else
             {
               yyparser.yylval = new CalcParserVal(yyparser.pushValue(memory.get(index)));
               return CalcParser.VALUE;
             }
         }
       }

{NAME} { moveCursor(); 
         yyparser.yylval = new CalcParserVal(yytext());
         return CalcParser.IDENTIFIER; }

{SPC}  { moveCursor(); }

[^]    { moveCursor();
         yyparser.yyerror("Bad character '" + yytext() + "'");
         return -1; }

